GlowScript 2.7 VPython

#Simulador do efeito Doppler para ondas mecânicas.
#Essa simulação precisa ser rodada em glowscript.org
#Para rodar diretamente em glowscript, acesse: https://www.glowscript.org/#/user/jocoteles/folder/Fisica2/program/DopplerEffect

scene.fov = 0.01
scene.range = 200

lbox = 4     #largura da frente de onda [m]
cbox = 300   #comprimento da frente de onda [m]
pbox = 60    #profundidade da frente de onda [m]
amp = cbox/8 #amplitude da onda

detx0 = 100   #posição inicial do detetor [m]
soux0 = -100  #posição inicial da fonte [m]

v = 10.0        #velocidade da onda [m/s]
T = 4.0         #período da onda da fonte [s]
dt = 0.1        #intervalo de tempo de amostragem [s]
w = 2*pi/T      #frequência angular da onda [rad/s]
#k = v/w         #número de onda [rad/m]
#dx = v*dt/10    #intervalo de tolerância para detecção da passagem da frente de onda [m]
N = int(T/dt)   #numero de amostragens de um período da onda da fonte
vmax = v        #módulo da velocidade máxima da fonte e do detector [m/s]
tint = 20*T     #intervalo temporal varrido pelo gráfico [s]
ear = 2         #largura do eixo x

rodar = True

def fw(x, c=color.yellow): #função que retorna uma frente de onda
    return box(pos=vec(x,0,0), size=vec(lbox,cbox,pbox), color=c, opacity = 0.5)
def yw(x, y, c=color.cyan):
    return sphere(pos=vec(x,y,0), radius = 3, color=c, opacity = 0.5)

def bPausar(b):
    global rodar
    rodar = not rodar
    if (b.text == 'Pausar'):
        b.text = 'Rodar'
    else:
        b.text = 'Pausar'
button(bind=bPausar, text='Pausar', pos=scene.title_anchor)        


arrow(pos=vec(-1.1*5*scene.range,0,0), axis=vec(1.1*10*scene.range,0,0), color=color.blue, shaftwidth = ear, headwidth = 3*ear, headlenght = 2*ear)
for i in range(-5*scene.range, 5*scene.range):
    if i%40 == 0:
        sphere(pos=vec(i,0,0),radius=1.5*ear,color=color.white)
        text(text=str(i)+'m', pos=vec(i,-16,0), axis=vec(1,0,0), align='center', height=8, depth=ear/2, color=color.white)
    if i%10 == 0:
        sphere(pos=vec(i,0,0),radius=ear,color=color.white)

det = box(pos=vec(detx0,0,0), size=vec(lbox,cbox,pbox), color=color.green)
sou = box(pos=vec(soux0,0,0), size=vec(lbox,cbox,pbox), color=color.red)
dettext = text(text='detetor', pos=vec(detx0,1.1*cbox/2,0), axis=vec(0,0,-1), align='center', height=cbox/10, depth=lbox, color=color.green)
soutext = text(text='fonte', pos=vec(soux0,1.1*cbox/2,0), axis=vec(0,0,-1), align='center', height=cbox/10, depth=lbox, color=color.red)
det.vel = 0
sou.vel = 0

def set_det_vel(b):
    det.vel = b.value
    det_vel_text.text = 'Velocidade detector = '+'{:1.1f}'.format(det_vel_slider.value)+' m/s\n'
def set_sou_vel(b):
    sou.vel = b.value
    sou_vel_text.text = 'Velocidade fonte = '+'{:1.1f}'.format(sou_vel_slider.value)+' m/s\n'

scene.append_to_caption('\n\n')

det_vel_slider = slider(min=-vmax, max=vmax, value=det.vel, bind=set_det_vel, right=15)
det_vel_text = wtext(text='Velocidade detector = '+'{:1.1f}'.format(det_vel_slider.value)+' m/s\n', pos=scene.caption_anchor)

sou_vel_slider = slider(min=-vmax, max=vmax, value=sou.vel, bind=set_sou_vel, right=15)
sou_vel_text = wtext(text='Velocidade fonte = '+'{:1.1f}'.format(sou_vel_slider.value)+' m/s\n', pos=scene.caption_anchor)

s = 'Forma de onda detectada.'
grafico = graph(title=s, xtitle='tempo [s]', ytitle='Amplitude [u.a.]',  fast=False, xmin=0, xmax=1, height=200)
curva = gcurve(width=2,color=color.red)

t = 0.0
i = 0
frontd, fronte = [], []
amplid, amplie = [], []
gflag = True
while True:
    if rodar:
        swt = sin(w*t)
        amplie.append(yw(sou.pos.x,amp*swt))
        amplid.append(yw(sou.pos.x,amp*swt))
        sou.size.x = lbox*(1.5+swt)
        if i%N == 0:    #geração de novas frentes de onda
            frontd.append(fw(sou.pos.x))
            fronte.append(fw(sou.pos.x))
        det.pos.x += det.vel*dt
        sou.pos.x += sou.vel*dt
        dettext.pos.x = det.pos.x
        soutext.pos.x = sou.pos.x
        for fd, fe in zip(frontd, fronte):        
            fd.pos.x += v*dt
            fe.pos.x -= v*dt
            if fd.pos.x > 5*scene.range:
                fd.visible = False
                del fd
            if fe.pos.x < -5*scene.range:
                fe.visible = False
                del fe
        for ad, ae in zip(amplid, amplie):
            pastd = ad.pos.x-det.pos.x
            paste = ae.pos.x-det.pos.x
            ad.pos.x += v*dt
            ae.pos.x -= v*dt
            if (pastd*(ad.pos.x-det.pos.x) <= 0):            
                curva.plot(t, ad.pos.y)
                if gflag:
                    grafico.xmin = t
                    grafico.xmax = t + tint
                    gflag = False
            if (paste*(ae.pos.x-det.pos.x) <= 0):
                curva.plot(t, ae.pos.y)
                if gflag:
                    grafico.xmin = t
                    grafico.xmax = t + tint
                    gflag = False            
            if ad.pos.x > 5*scene.range:
                ad.visible = False
                del ad
            if ae.pos.x < -5*scene.range:
                ae.visible = False
                del ae
        if t > grafico.xmax:
            d = t-grafico.xmax
            grafico.xmin += d
            grafico.xmax += d
        t += dt
        i += 1
    sleep(dt)
    
    
